%function husimi(N,pow,delay,map,eigen)
function husimi

%if nargin < 5, eigen = 0; end;
%if nargin < 4, map = gray; end;
%if nargin < 3, delay = 1; end;
%if nargin < 2, pow = 1; end;
%if nargin < 1, N = 131; end;

% Make these variables global so that callback scripts can see them when
% needed (maybe not the best way to do this, but it gets the job done...)
global count vec init V samematrix N pow delay map U samematrix sigma x ex;

% Initialize parameters.
N = 200;
pow = 1;
delay = 1;
map = gray;
halt = true;
forward = 1;
done = 0;

% Create generic functions
ex = @(x,sig,shift) exp(-(x-shift).^2/(2*sig^2));
exi = @(x,sig,shift) exp(-1i*(x-shift).^2/(2*sig^2));
si = @(x,period,shift) sin(2*pi*(x-shift)./period);
sii = @(x,period,shift) 1i*sin(2*pi*(x-shift)./period);

    % Create a torus
    function [x,y,z] = torus()
        a=5;
        c=10;
        [u,v]=meshgrid(0:360/(N-1):360);
        x=(c+a*cosd(v)).*cosd(u);
        y=(c+a*cosd(v)).*sind(u);
        z=a*sind(v);
    end

    % Pause user interface
    function stop(in1, in2)
        halt = ~halt;
        if halt, uiwait; end;
        if ~halt, uiresume; end;
    end
    
    % Swap the matrix with its inverse to "reverse time"
    function reverse(in1, in2)
        forward = ~forward;
        U = U';
        halt = false;
        uiresume;
    end

    % Kill the applet
    function terminate(in1, in2)
        uiresume;
        samematrix=0;
        done=1;
    end

while ~done
    dx = 1/N;
    x = (1:N)*dx - 1/2;
    sigma = 1/sqrt(N)/2.5;
    U = zeros(N,N);
    count = 0;
    hold off;
    az = -37.5;
    el = 30;
    samematrix=1;
    X = zeros(N,N); % matrix of windowed columns
    [xx,y,z] = torus();
    
    % Generate unitary transformation matrix
    for j=1:N
        for k=1:N
            U(k,j) = sqrt(1i/N)*exp((2*pi*1i/N)*((j+N/2)^2-(j+N/2)*(k+N/2)+(k+N/2)^2));
        end
    end
    [V D] = eig(U);
    U = U^pow;
    
    % Normalize initial vector
    init = ex(x,sigma,0);
    init = init / norm(init,2);

    % Copy contents of init into vec
    vec = init;

    % Create figure handle
    hfig = gcf;
    set(hfig, 'name', 'Husimi');
    set(hfig, 'numbertitle', 'off');
    hcmenu = uicontextmenu;
    uimenu(hcmenu, 'Label', 'Start/Pause Animation', 'Callback', @stop);
    uimenu(hcmenu, 'Label', 'Change Direction', 'Callback', @reverse);

    % Construct time step menu
    timestep = uimenu(hcmenu, 'Label', 'Time Step');
    steps = [ 1 1/2 1/3 1/4 1/8 1/10 1/15 1/20 1/25 1/50 1/100 ];
    for j=1:length(steps)
        timestep_call = ['timestep_action=' num2str(steps(j)) '; timestep_control;'];
        uimenu(...
            timestep,...
            'Label', num2str(steps(j)),...
            'Callback', timestep_call...
        );
    end
    
    % Construct frame rate menu
    framerate = uimenu(hcmenu, 'Label', 'Frames Per Second');
    rates = [ 1 2 4 10 15 20 24 29.96 50 ];
    for j=1:length(rates)
        framerate_call = ['framerate_action=' num2str(rates(j)) '; framerate_control;'];
        uimenu(...
            framerate,...
            'Label', num2str(rates(j)),...
            'Callback', framerate_call...
        );
    end

    set(hfig, 'uicontextmenu', hcmenu);
    
    % Default Initial Wave Packets Menu
    waveparent = uimenu(hcmenu, 'Label', 'Initial Vectors');
    wave_call = ['wave_action=1; wave_control;'];
    uimenu(waveparent, 'Label', 'Gaussian (offset: 0)', 'Callback', wave_call);

    % Construct Parent Eigenvector Menu
    size = 25;
    eigenparent = uimenu(hcmenu, 'Label', 'Eigenvectors');
    for j=1:ceil(N/size)
        % Construct Child Eigenvector Menus
        eigenchild = uimenu(eigenparent, 'Label', sprintf('Eigenvectors %d-%d', (j-1)*size+1, min(N,j*size)));
        for k=1:size
            index = (j-1)*size+k;
            if index <= N
                eigenvector_call = ['eigenvector_action=' int2str(index) '; eigenvector_control;'];
                uimenu(...
                    eigenchild,...
                    'Label', sprintf('#%d (%.3f,%.3fi)', index, real(D(index,index)), imag(D(index,index))),...
                    'Callback', eigenvector_call...
                );
            end
        end
    end
    
    % Generate Matrix Menu
    maxdim = 500;
    matrixparent = uimenu(hcmenu, 'Label', 'Dimension');
    for j=1:ceil(maxdim/size)
        matrixchild = uimenu(matrixparent, 'Label', sprintf('%d-%d', (j-1)*size+1, min(maxdim, j*size)));
        for k=1:size
            index = (j-1)*size+k;
            if index <= maxdim
                matrix_call = ['matrix_action=' int2str(index) '; matrix_control;'];
                uimenu(...
                    matrixchild,...
                    'Label', sprintf('%d', index),...
                    'Callback', matrix_call...
                );
            end
        end
    end

    uimenu(hcmenu, 'Label', 'Quit', 'Callback', @terminate);

    % As long as we haven't altered the animation too drastically, continue
    % as usual
    while samematrix == 1
        % Apply windows vector to our current vector
        for j=1:N
            % Generate windowed vector
            window = exp(-(x-x(j)).^2 / (2 * sigma^2)); % translated window
            window = window+exp(-(x-x(j)-1).^2 / (2 * sigma^2));
            window = window+exp(-(x-x(j)+1).^2 / (2 * sigma^2));
            plot(1:N, window);

            product = vec.*window; % product of init and wind
            X(:,j) = product; % add product to matrix to be transformed
        end
        M = fftshift(fft(X,N),1); % fft the columns of this matrix

        % Main Husimi plot
        subplot(15,1,1:8);
        str = sprintf('Frames Per Second = %.2f; N = %d; Time Step = %.2f', 1/delay, N, pow);
        image(x, x, abs(M), 'CDataMapping', 'scaled');
        axis square;
        colormap(map);
        title(str);

        % Real/Imaginary signals
        subplot(15,1,9:10);
        plot(x, [real(vec); imag(vec)], '-');
        axis([-0.5 0.5 -0.3 0.3 ]);

        % Torus plot
        subplot(15,1,11:15);
        surf(xx, y, z, abs(M));
        colormap(map);
        shading flat;
        axis equal;
        view([az el]);

        % Pause for a specified amount of time, and store torus camera data
        pause(delay);
        [az, el] = view;

        % Terminate if when we recover the original signal
%        if norm(vec-init, 2) < 1e-9 && count ~= 0 && forward == 1, return; end;

        % Act matrix on vector and increment the count
        vec = (U*vec')';
        count = count + 1;
        
        % If we've reset the matrix and the halt flag is set, pause the UI
        if count == 1 && halt==true
            uiwait;
        end
    end
end

end
